/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * OutputAsAction.cpp
 *
 *  Created on: May 8, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Parser/FormatParserStorage.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include <boost/filesystem/fstream.hpp>

#include "Actions/WorldAction/OutputAsAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "OutputAsAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr WorldOutputAsAction::performCall() {
  LOG(1, "Storing world to file " << params.filename.get() << ".");

  // extract suffix
  std::string FilenameSuffix;
  std::string FilenamePrefix;
  if (params.filename.get().has_filename()) {
    // get suffix
    FilenameSuffix = params.filename.get().extension().string().substr(1); // remove the prefixed "."
    FilenamePrefix = params.filename.get().stem().string();
  } else {
    STATUS("Output file does not have a suffix, cannot recognize format.");
    return Action::failure;
  }

  // store file
  // parse the file
  boost::filesystem::ofstream output;
  output.open(params.filename.get());
  if (!output.fail()) {
    FormatParserStorage::getInstance().saveWorld(output, FilenameSuffix);
  } else {
    STATUS("Could not open file "+params.filename.get().string()+".");
    return Action::failure;
  }
  output.close();

  return Action::success;
}

ActionState::ptr WorldOutputAsAction::performUndo(ActionState::ptr _state) {
//  ParserLoadXyzState *state = assert_cast<ParserLoadXyzState*>(_state.get());

  STATUS("Undo of WorldOutputAsAction not implemented.");
  return Action::failure;
//  string newName = state->mol->getName();
//  state->mol->setName(state->lastName);
//
//  return ActionState::ptr(new ParserLoadXyzState(state->mol,newName));
}

ActionState::ptr WorldOutputAsAction::performRedo(ActionState::ptr _state){
  STATUS("Redo of WorldOutputAsAction not implemented.");
  return Action::failure;
}

bool WorldOutputAsAction::canUndo() {
  return false;
}

bool WorldOutputAsAction::shouldUndo() {
  return false;
}
/** =========== end of function ====================== */
